/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.datamodel;

import static org.testng.Assert.assertTrue;

import java.util.NoSuchElementException;

import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class VisibleColsIteratorTest
{
  HiddenColumns hiddenCols;

  HiddenColumns hiddenColsAtStart;

  @BeforeClass(groups = { "Functional" })
  public void setup()
  {
    hiddenCols = new HiddenColumns();
    hiddenCols.hideColumns(2, 4);

    hiddenColsAtStart = new HiddenColumns();
    hiddenColsAtStart.hideColumns(0, 2);
  }

  /*
   * Test iterator iterates correctly through the columns
   * when alignment has hidden cols
   */
  @Test(groups = { "Functional" })
  public void testHasNextAndNextWithHidden()
  {
    VisibleColsIterator it = new VisibleColsIterator(0, 6, hiddenCols);
    int count = 0;
    while (it.hasNext())
    {
      it.next();
      count++;
    }
    assertTrue(count == 4, "hasNext() is false after 4 iterations");
  }

  /*
   * Test iterator iterates correctly through the columns
   * when alignment has no hidden cols
   */
  @Test(groups = { "Functional" })
  public void testHasNextAndNextNoHidden()
  {
    VisibleColsIterator it2 = new VisibleColsIterator(0, 3,
            new HiddenColumns());
    int count = 0;
    while (it2.hasNext())
    {
      it2.next();
      count++;
    }
    assertTrue(count == 4, "hasNext() is false after 4 iterations");
  }

  /*
   * Test iterator iterates correctly through the columns
   * when alignment has hidden cols at start
   */
  @Test(groups = { "Functional" })
  public void testHasNextAndNextStartHidden()
  {
    VisibleColsIterator it3 = new VisibleColsIterator(0, 6,
            hiddenColsAtStart);
    int count = 0;
    while (it3.hasNext())
    {
      it3.next();
      count++;
    }
    assertTrue(count == 4, "hasNext() is false after 4 iterations");
  }

  /*
   * Test iterator iterates correctly through the columns
   * when alignment has hidden cols at end
   */
  @Test(groups = { "Functional" })
  public void testHasNextAndNextEndHidden()
  {
    VisibleColsIterator it4 = new VisibleColsIterator(0, 4, hiddenCols);
    int count = 0;
    while (it4.hasNext())
    {
      it4.next();
      count++;
    }
    assertTrue(count == 2, "hasNext() is false after 2 iterations");

  }

  /*
   * Test iterator always throws NoSuchElementException at end of iteration
   * when alignment has hidden cols
   */
  @Test(
    groups = { "Functional" },
    expectedExceptions = { NoSuchElementException.class })
  public void testLastNextWithHidden() throws NoSuchElementException
  {
    VisibleColsIterator it = new VisibleColsIterator(0, 3, hiddenCols);
    while (it.hasNext())
    {
      it.next();
    }
    it.next();
  }

  /*
   * Test iterator always throws NoSuchElementException at end of iteration
   * when alignment has no hidden cols
   */
  @Test(
    groups = { "Functional" },
    expectedExceptions = { NoSuchElementException.class })
  public void testLastNextNoHidden() throws NoSuchElementException
  {
    VisibleColsIterator it2 = new VisibleColsIterator(0, 3,
            new HiddenColumns());
    while (it2.hasNext())
    {
      it2.next();
    }
    it2.next();
  }

  /*
   * Test iterator always throws NoSuchElementException at end of iteration
   * when alignment has hidden cols at start
   */
  @Test(
    groups = { "Functional" },
    expectedExceptions = { NoSuchElementException.class })
  public void testLastNextStartHidden() throws NoSuchElementException
  {
    VisibleColsIterator it3 = new VisibleColsIterator(0, 6,
            hiddenColsAtStart);
    while (it3.hasNext())
    {
      it3.next();
    }
    it3.next();
  }

  /*
   * Test iterator always throws NoSuchElementException at end of iteration
   * when alignment has hidden cols at end
   */
  @Test(
    groups = { "Functional" },
    expectedExceptions = { NoSuchElementException.class })
  public void testLastNextEndHidden() throws NoSuchElementException
  {
    VisibleColsIterator it4 = new VisibleColsIterator(0, 4, hiddenCols);
    while (it4.hasNext())
    {
      it4.next();
    }
    it4.next();
  }

  /*
   * Test calls to remove throw UnsupportedOperationException
   */
  @Test(
    groups = { "Functional" },
    expectedExceptions = { UnsupportedOperationException.class })
  public void testRemove() throws UnsupportedOperationException
  {
    VisibleColsIterator it = new VisibleColsIterator(0, 3, hiddenCols);
    it.remove();
  }
}
